#ifndef EXPRESION_H
#define EXPRESION_H

#include "parser_aux.h"
#include "visitante.h"
#include <vector>
using namespace std;


struct expresion {
   virtual void visita(visitante*) = 0;
};

struct expresion_identificador : expresion {
   token_anotada* identificador;
   
   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct expresion_literal : expresion {
   token_anotada* literal;
   
   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct expresion_prefija : expresion {
   token_anotada* operador;
   expresion* sobre;
   
   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct expresion_binaria : expresion {
   expresion* izq;
   token_anotada* operador;
   expresion* der;

   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct expresion_llamada_funcion : expresion {
   expresion* sobre;
   vector<expresion*> parametros;

   void visita(visitante* v)
   {
      v->procesa(this);
   }
};


expresion* parser_expresion(token_anotada*&);

vector<expresion*> parser_lista_parametros(token_anotada*& entrada)
{
   vector<expresion*> parametros;
   return parametros;
}

expresion* parser_expresion_primaria(token_anotada*& entrada)
{
   if (entrada->valor == IDENTIFICADOR) {
      expresion_identificador* ex = new expresion_identificador( );
      ex->identificador = entrada++;

      return ex;
   }
   else if (entrada->valor == LITERAL_ENTERA) {
      expresion_literal* ex = new expresion_literal( );
      ex->literal = entrada++;

      return ex;
   }
   else {
      espera(entrada, PARENTESIS_IZQ);
      expresion* ex = parser_expresion(entrada);
      espera(entrada, PARENTESIS_DER);
      
      return ex;
   }
}

expresion* parser_expresion_unaria(token_anotada*& entrada)
{
   if (es_operador_prefijo(entrada->valor)) {
      expresion_prefija* ex = new expresion_prefija( );
      
      ex->operador = entrada++;
      ex->sobre = parser_expresion_unaria(entrada);

      return ex;
   }
   
   expresion* ex = parser_expresion_primaria(entrada);
   
   if (entrada->valor == PARENTESIS_IZQ) {
      expresion_llamada_funcion* ef = new expresion_llamada_funcion( );
      
      ef->sobre = ex;
      ef->parametros = parser_lista_parametros(++entrada);
      ex = ef;
      
      espera(entrada, PARENTESIS_DER);
   }
   
   return ex;
}

expresion* parser_expresion_n_aria(token_anotada*& entrada, int prec)
{
   expresion* ex = parser_expresion_unaria(entrada);

   while (es_operador_binario(entrada->valor) && precedencia(entrada->valor) >= prec) {
      expresion_binaria* eb = new expresion_binaria( );
      
      eb->izq = ex;
      eb->operador = entrada++;
      eb->der = parser_expresion_n_aria(entrada, precedencia(eb->operador->valor) + asociatividad(eb->operador->valor));
      
      ex = eb;
   }

   return ex;
}

expresion* parser_expresion(token_anotada*& entrada)
{
   return parser_expresion_n_aria(entrada, 0);
}

#endif
