#ifndef LEXER_H
#define LEXER_H

#include "error.h"
#include <cctype>
#include <cstdio>
#include <vector>
using namespace std;


enum token {
   INCLUDE,          // 0
   FUNCTION,         // 1
   VAR,              // 2
   IF,               // 3
   ELSE,             // 4
   RETURN,           // 5
   PRINT,            // 6
   MAS,              // 7
   MENOS,            // 8
   POR,              // 9
   ENTRE,            // 10
   ASIGNACION,       // 11
   MENOR_QUE,        // 12
   MENOR_IGUAL_QUE,  // 13
   MAYOR_QUE,        // 14
   MAYOR_IGUAL_QUE,  // 15
   IGUAL_QUE,        // 16
   DIFERENTE_QUE,    // 17
   PARENTESIS_IZQ,   // 18
   PARENTESIS_DER,   // 19
   LLAVE_IZQ,        // 20
   LLAVE_DER,        // 21
   COMA,             // 22
   PUNTO_COMA,       // 23
   LITERAL_ENTERA,   // 24
   LITERAL_CADENA,   // 25
   IDENTIFICADOR,    // 26
   DESCONOCIDO,      // 27
   FIN_ARCHIVO       // 28
};

struct token_anotada {
   token valor;
   
   char* inicio;
   int tam; 
};

token_anotada crea_token_anotada(token v)
{
   token_anotada ta;
   ta.valor = v;
   
   return ta;
}

token_anotada crea_token_anotada(token v, char* i, int t)
{
   token_anotada ta;
   ta.valor = v;
   ta.inicio = i;
   ta.tam = t;
   
   return ta;
}

bool continua_id(char c)
{
   return isalnum(c) || c == '_';
}

bool es_asignacion(char* actual)
{
   return *actual == '=';
}

bool es_igual(char* actual)
{
   return *actual == '=' && actual[1] == '=';
}

bool es_mas(char* actual)
{
   return *actual == '+';
}

bool es_por(char* actual)
{
   return *actual == '*';
}

bool es_parentesis_izq(char* actual)
{
   return *actual == '(';
}

bool es_parentesis_der(char* actual)
{
   return *actual == ')';
}

bool es_llave_izq(char* actual)
{
   return *actual == '{';
}

bool es_llave_der(char* actual)
{
   return *actual == '}';
}

bool es_coma(char* actual)
{
   return *actual == ',';
}

bool es_punto_coma(char* actual)
{
   return *actual == ';';
}

bool es_if(char* actual)
{
   return *actual == 'i' && actual[1] == 'f' && !continua_id(actual[2]);
}

bool es_else(char* actual)
{
   return *actual == 'e' && actual[1] == 'l' && actual[2] == 's' && actual[3] == 'e' && !continua_id(actual[4]);
}

bool es_include(char* actual)
{
   return *actual == 'i' & actual[1] == 'n' && actual[2] == 'c' && actual[3] == 'l' && actual[4] == 'u' && actual[5] == 'd' && actual[6] == 'e' && !continua_id(actual[7]);
}

bool es_function(char* actual)
{
   return *actual == 'f' && actual[1] == 'u' && actual[2] == 'n' && actual[3] == 'c' && actual[4] == 't' && actual[5] == 'i' && actual[6] == 'o' && actual[7] == 'n' && !continua_id(actual[8]);
}

bool es_var(char* actual)
{
   return *actual == 'v' && actual[1] == 'a' && actual[2] == 'r' && !continua_id(actual[3]);
}

bool es_literal_entera(char* actual, int& tam)
{
   tam = 0;
   
   while (isdigit(*actual)) {
     ++actual;
     ++tam;
   }
   
   return tam > 0;
}

bool es_literal_cadena(char* actual, int& tam)
{
   tam = 2;
   
   if (*actual == '"') {
     ++actual;
     
     while (*actual != '"') {
       if (*actual == EOF) {
         return false;
       }
       
       ++actual;
       ++tam;
     }
     
     return true;
   }
   
   return false;
}

bool es_identificador(char* actual, int& tam)
{
   if (isalpha(*actual) || *actual == '_') {
     tam = 0;
      
     do {
       ++actual;
       ++tam;
     } while (continua_id(*actual));
     
     return true;
   }
   
   return false;
}

vector<token_anotada> lexer(char* actual)
{
   vector<token_anotada> tokens;
   int tam;
   
   for (;;) {
     while (isspace(*actual)) {
       ++actual;
     }
     
     if (*actual == EOF) {
       tokens.push_back(crea_token_anotada(FIN_ARCHIVO));
       break;
     }
     else if (es_igual(actual)) {
       tokens.push_back(crea_token_anotada(IGUAL_QUE));
       actual += 2;
     }
     else if (es_asignacion(actual)) {
       tokens.push_back(crea_token_anotada(ASIGNACION));
       ++actual;
     }
     else if (es_mas(actual)) {
       tokens.push_back(crea_token_anotada(MAS));
       ++actual;
     }
     else if (es_por(actual)) {
       tokens.push_back(crea_token_anotada(POR));
       ++actual;
     }
     else if (es_parentesis_izq(actual)) {
       tokens.push_back(crea_token_anotada(PARENTESIS_IZQ));
       ++actual;
     }
     else if (es_parentesis_der(actual)) {
       tokens.push_back(crea_token_anotada(PARENTESIS_DER));
       ++actual;
     }
     else if (es_llave_izq(actual)) {
       tokens.push_back(crea_token_anotada(LLAVE_IZQ));
       ++actual;
     }
     else if (es_llave_der(actual)) {
       tokens.push_back(crea_token_anotada(LLAVE_DER));
       ++actual;
     }
     else if (es_coma(actual)) {
       tokens.push_back(crea_token_anotada(COMA));
       ++actual;
     }
     else if (es_punto_coma(actual)) {
       tokens.push_back(crea_token_anotada(PUNTO_COMA));
       ++actual;
     }
     else if (es_if(actual)) {
       tokens.push_back(crea_token_anotada(IF));
       actual += 2;
     }
     else if (es_else(actual)) {
       tokens.push_back(crea_token_anotada(ELSE));
       actual += 4;
     }
     else if (es_include(actual)) {
       tokens.push_back(crea_token_anotada(INCLUDE));
       actual += 7;   
     }
     else if (es_function(actual)) {
       tokens.push_back(crea_token_anotada(FUNCTION));
       actual += 8;   
     }
     else if (es_var(actual)) {
       tokens.push_back(crea_token_anotada(VAR));
       actual += 3;
     }
     else if (es_literal_entera(actual, tam)) {
       tokens.push_back(crea_token_anotada(LITERAL_ENTERA, actual, tam));
       actual += tam;
     }
     else if (es_literal_cadena(actual, tam)) {
       tokens.push_back(crea_token_anotada(LITERAL_CADENA, actual, tam));
       actual += tam;
     }
     else if (es_identificador(actual, tam)) {
       tokens.push_back(crea_token_anotada(IDENTIFICADOR, actual, tam));
       actual += tam;
     }
     else {
       tokens.push_back(crea_token_anotada(DESCONOCIDO));
       break;
     }
   }
   
   return tokens;
}

#endif
