#ifndef SENTENCIA_H
#define SENTENCIA_H

#include "expresion.h"
#include "parser_aux.h"
#include "visitante.h"
#include <vector>
using namespace std;


struct sentencia {
   virtual void visita(visitante*) = 0;
};

struct sentencia_declaracion : sentencia {
   token_anotada* nombre;
   expresion* inicializador;

   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct sentencia_if : sentencia {
   expresion* condicion;
   vector<sentencia*> parte_si;
   vector<sentencia*> parte_no;

   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct sentencia_return : sentencia {
   expresion* valor;

   void visita(visitante* v)
   {
      v->procesa(this);
   }
};

struct sentencia_print : sentencia {
   expresion* valor;
   
   void visita(visitante* v)
   {
      v->procesa(this);
   }
};


sentencia* parser_sentencia(token_anotada*& entrada)
{
   if (entrada->valor == IF) {
      sentencia_if* sen = new sentencia_if( );
      sen->condicion = parser_expresion(++entrada);
      espera(entrada, LLAVE_IZQ);
      
      while (entrada->valor != LLAVE_DER) {
         sen->parte_si.push_back(parser_sentencia(entrada));   
      }
      
      ++entrada;
      
      if (entrada->valor == ELSE) {
         ++entrada;
         
         if (entrada->valor == IF) {
            sen->parte_no.push_back(parser_sentencia(entrada));
         }
         else {
            espera(entrada, LLAVE_IZQ);
            
            while (entrada->valor != LLAVE_DER) {
               sen->parte_no.push_back(parser_sentencia(entrada));   
            }
            
            ++entrada;
         }
      }
      
      return sen;
   }
   else if (entrada->valor == RETURN) {
      error("NO IMPLEMENTADO");
   }
   else if (entrada->valor == PRINT) {
      error("NO IMPLEMENTADO");
   }
   else {
      sentencia_declaracion* sen = new sentencia_declaracion( );
      
      espera(entrada, VAR);
      espera(entrada, IDENTIFICADOR);
      espera(entrada, ASIGNACION);
      
      sen->nombre = entrada - 2;
      sen->inicializador = parser_expresion(entrada);
      espera(entrada, PUNTO_COMA);
      
      return sen;
   }
}

#endif
