#ifndef VISITANTE_IMPRESOR_H
#define VISITANTE_IMPRESOR_H

#include "expresion.h"
#include "sentencia.h"
#include "visitante.h"
#include <iostream>
using namespace std;

      
struct visitante_impresor : visitante {
   int sangria;

   void agrega_salida(int n)
   {
      cout << n;
   }

   void agrega_salida(char c)
   {
      cout << c;
   }
   
   void agrega_salida(char c, int n)
   {
      for (int i = 0; i < n; ++i) {
         cout << c;
      }
   }
      
   void agrega_salida(const char* c)
   {
      cout << c;
   }
   
   void agrega_salida(token t)
   {
      if (t == MAS) {
         cout << '+';
      }
      else if (t == POR) {
         cout << '*';
      }
      else if (t == ASIGNACION) {
         cout << '=';
      }
      else if (t == IGUAL_QUE) {
         cout << "==";
      }
   }
   
   void agrega_salida(token_anotada* t)
   {
      if (t->valor == IDENTIFICADOR || t->valor == LITERAL_CADENA || t->valor == LITERAL_ENTERA) {
         cout.write(t->inicio, t->tam);
      }
      else {
         agrega_salida(t->valor);
      }
   }

   void agrega_salida(sentencia* nodo)
   {
      sangria += 4;
      nodo->visita(this);
      sangria -= 4;
   }
   
   void agrega_salida(expresion* nodo)
   {
      sangria += 4;
      nodo->visita(this);
      sangria -= 4;
   }
   

   visitante_impresor( )
   {
      sangria = 0;
   }
   
   void agrega_salida(arbol_sintactico* nodo)
   {
      for (int i = 0; i < nodo->inclusiones.size( ); ++i) {
         agrega_salida("include ");
         agrega_salida(nodo->inclusiones[i]);
         agrega_salida(";\n");
      }
      
      agrega_salida('\n');
      
      for (int i = 0; i < nodo->funciones.size( ); ++i) {
         agrega_salida(&nodo->funciones[i]);
      }
   }
   
   void agrega_salida(funcion* nodo)
   {
      agrega_salida("function ");
      agrega_salida(nodo->nombre);
      agrega_salida("(");
      
      for (int i = 0; i < nodo->parametros.size( ); ++i) {
         agrega_salida(nodo->parametros[i]);
         agrega_salida(", ");
      }
      
      agrega_salida(")\n{\n");

      for (int i = 0; i < nodo->instrucciones.size( ); ++i) {
         agrega_salida(nodo->instrucciones[i]);
      }
      
      agrega_salida("}\n\n");
   }
   
   void procesa(sentencia_declaracion* nodo)
   {
      agrega_salida(' ', sangria);
      agrega_salida("var ");
      agrega_salida(nodo->nombre);
      agrega_salida(" = ");
      agrega_salida(nodo->inicializador);
      agrega_salida(";\n");
   }
   
   void procesa(sentencia_if* nodo)
   {
      agrega_salida(' ', sangria);
      agrega_salida("if ");
      agrega_salida(nodo->condicion);
      agrega_salida(" {\n");
      
      for (int i = 0; i < nodo->parte_si.size( ); ++i) {
         agrega_salida(nodo->parte_si[i]);
      }
      
      agrega_salida(' ', sangria);
      agrega_salida("}\n");
      agrega_salida(' ', sangria);
      agrega_salida("else {\n");
      
      for (int i = 0; i < nodo->parte_no.size( ); ++i) {
         agrega_salida(nodo->parte_no[i]);
      }
      
      agrega_salida(' ', sangria);
      agrega_salida("}\n");
   }
      
   void procesa(expresion_identificador* nodo)
   {
      agrega_salida(nodo->identificador);
   }
   
   void procesa(expresion_literal* nodo)
   {
      agrega_salida(nodo->literal);
   }
   
   void procesa(expresion_prefija* nodo)
   {
      agrega_salida(nodo->operador);
      agrega_salida('(');
      agrega_salida(nodo->sobre);
      agrega_salida(')');
   }
   
   void procesa(expresion_binaria* nodo)
   {
      agrega_salida('(');
      agrega_salida(nodo->izq);
      agrega_salida(')');
      agrega_salida(nodo->operador);
      agrega_salida('(');
      agrega_salida(nodo->der);
      agrega_salida(')');
   }
   
   void procesa(expresion_llamada_funcion* nodo)
   {
      agrega_salida('(');
      agrega_salida(nodo->sobre);
      agrega_salida(')');
      agrega_salida('(');
      
      for (int i = 0; i < nodo->parametros.size( ); ++i) {
         agrega_salida('(');
         agrega_salida(nodo->parametros[i]);
         agrega_salida("),");
      }
      
      agrega_salida(')');
   }
};

#endif
